@@ -29,7 +29,7 @@ from random import random
 from fluids.constants import *
 from fluids.numerics import assert_close, assert_close1d, assert_close2d, assert_close3d
 from numpy.testing import assert_allclose
-from chemicals import normalize
+from chemicals import normalize, Rackett_fit, Antoine
 from thermo.test_utils import check_np_output_activity
 
 import pytest
@@ -372,7 +372,28 @@ def test_volume_numba_solvers():
     assert slns[1] == 0
     assert slns[2] == 0
     assert_close(slns[0], 2.5908397553496098e-05, rtol=1e-15)
+
+from .test_eos_volume import hard_parameters, validate_volume
+@pytest.mark.parametrize("params", hard_parameters)
+@mark_as_numba
+def test_hard_default_solver_volumes_numba(params):
+    # Particularly tough cases
+    validate_volume(params, thermo.numba.eos_volume.volume_solutions_halley, rtol=1e-14)    
+
+
+from .test_flash_pure import test_V_error_plot, pure_fluids, eos_list
+@pytest.mark.slow
+@pytest.mark.plot
+@pytest.mark.parametric
+@pytest.mark.parametrize("fluid", pure_fluids)
+@pytest.mark.parametrize("eos", eos_list)
+@pytest.mark.parametrize("P_range", ['high', 'low'])
+@mark_as_numba
+def test_V_error_plot_numba(fluid, eos, P_range):
+    return test_V_error_plot(fluid=fluid, eos=eos, P_range=P_range, solver=staticmethod(thermo.numba.eos_volume.volume_solutions_halley))
+
     
+
 @mark_as_numba
 def test_numba_dri_air():
     gas = thermo.numba.DryAirLemmon(T=300.0, P=1e5)
@@ -418,4 +439,23 @@ def test_lnphis_direct_and_sequential_substitution_2P_functional():
                                        trivial_solution_tol=1e-5, V_over_F_guess=0.5)
     assert_close(VF_calc, VF_expect, rtol=1e-6)
     assert_close1d(xs_calc, xs_expect)
-    assert_close1d(ys_calc, ys_expect)
\ No newline at end of file
+    assert_close1d(ys_calc, ys_expect)
+    
+    
+@mark_as_numba
+def test_fit_T_dep_numba():
+    Tc, rhoc, b, n, MW = 545.03, 739.99, 0.3, 0.28571, 105.921
+    Ts = np.linspace(331.15, 332.9, 10)
+    props_calc = [Rackett_fit(T, Tc, rhoc, b, n, MW) for T in Ts]
+    res, stats = TDependentProperty.fit_data_to_model(Ts=Ts, data=props_calc, model='Rackett_fit',
+                          do_statistics=True, use_numba=True, model_kwargs={'MW':MW, 'Tc': Tc,},
+                          fit_method='lm')
+    assert stats['MAE'] < 1e-5
+
+    # Numba was evaluating a 0 division when it shouldn't have, try excepted it
+    Ts = np.linspace(845.15, 1043.1999999999998, 20)
+    props_calc = [Antoine(T, A=1.9823770201329391, B=651.916, C=-1248.487) for T in Ts]
+    res, stats = TDependentProperty.fit_data_to_model(Ts=Ts, data=props_calc, model='Antoine',
+                          do_statistics=True, use_numba=True, model_kwargs={'base':10.0},
+                          fit_method='lm')
+    assert stats['MAE'] < 1e-5
