@@ -26,9 +26,12 @@ import pandas as pd
 import json
 from math import isnan
 from fluids.numerics import linspace, assert_close, derivative, assert_close1d
+from chemicals.vapor_pressure import *
 from thermo.vapor_pressure import *
 from thermo.vapor_pressure import SANJARI, EDALAT, AMBROSE_WALTON, LEE_KESLER_PSAT, BOILING_CRITICAL, COOLPROP, VDI_PPDS, VDI_TABULAR, WAGNER_MCGARRY, ANTOINE_EXTENDED_POLING, ANTOINE_POLING, WAGNER_POLING, DIPPR_PERRY_8E
+from thermo.utils import TDependentProperty
 from chemicals.identifiers import check_CAS
+import chemicals
 from thermo.coolprop import has_CoolProp
 from math import *
 
@@ -127,6 +130,178 @@ def test_VaporPressure():
     obj = VaporPressure(CASRN="71-43-2", Tb=353.23, Tc=562.05, Pc=4895000.0, omega=0.212, extrapolation="AntoineAB|DIPPR101_ABC", method="WAGNER_MCGARRY")
     assert_close(obj.T_dependent_property_derivative(600.0), 2379682.4349338813, rtol=1e-4)
 
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting0():
+    obj = VaporPressure(CASRN='13838-16-9')
+    Tmin, Tmax = obj.WAGNER_POLING_Tmin, obj.WAGNER_POLING_Tmax
+    Ts = linspace(Tmin, Tmax, 10)
+    Ps = [obj(T) for T in Ts]
+    Tc, Pc = obj.WAGNER_POLING_Tc, obj.WAGNER_POLING_Pc
+    fitted = obj.fit_data_to_model(Ts=Ts, data=Ps, model='Wagner', use_numba=False,
+                          model_kwargs={'Tc': obj.WAGNER_POLING_Tc, 'Pc': obj.WAGNER_POLING_Pc})
+    res = fitted
+    assert 'Tc' in res
+    assert 'Pc' in res
+    assert_close(res['a'], obj.WAGNER_POLING_coefs[0])
+    assert_close(res['b'], obj.WAGNER_POLING_coefs[1])
+    assert_close(res['c'], obj.WAGNER_POLING_coefs[2])
+    assert_close(res['d'], obj.WAGNER_POLING_coefs[3])
+
+    # Heavy compound fit
+    Ts = linspace(179.15, 237.15, 5)
+    props_calc = [Antoine(T, A=138., B=520200.0, C=3670.0) for T in Ts]
+    res, stats = TDependentProperty.fit_data_to_model(Ts=Ts, data=props_calc, model='Antoine',
+                          do_statistics=True, use_numba=False, model_kwargs={'base':10.0},
+                          fit_method='lm')
+    assert stats['MAE'] < 1e-5
+    
+    # Fit with very low range and no C
+    Ts = linspace(374, 377.0, 5)
+    props_calc = [Antoine(T, A=12.852103, B=2942.98, C=0.0) for T in Ts]
+    res, stats = TDependentProperty.fit_data_to_model(Ts=Ts, data=props_calc, model='Antoine',
+                          do_statistics=True, use_numba=False, model_kwargs={'base':10.0},
+                          fit_method='lm')
+    assert stats['MAE'] < 1e-5
+
+
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting1():
+    # Ammonia data fitting from chemsep
+    ammonia_Ts_Psats = [191.24, 202.546, 213.852, 225.157, 236.463, 239.82, 247.769, 259.075, 270.381, 281.686, 292.992, 304.298, 315.604, 326.909, 338.215, 349.521, 360.827, 372.133, 383.438, 394.744, 406.05]
+    ammonia_Psats = [4376.24, 10607.70, 23135.70, 46190.70, 85593.30, 101505.00, 148872.00, 245284.00, 385761.00, 582794.00, 850310.00, 1203550.00, 1658980.00, 2234280.00, 2948340.00, 3821410.00, 4875270.00, 6133440.00, 7621610.00, 9367940.00, 11403600.00]
+    res, stats = VaporPressure.fit_data_to_model(Ts=ammonia_Ts_Psats, data=ammonia_Psats, model='DIPPR101',
+                          do_statistics=True, use_numba=False, fit_method='lm')
+    assert stats['MAE'] < 1e-4
+
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting2_dippr():
+    pts = 10
+    fit_issue_CASs = ['75-07-0', '107-02-8', '108-38-3', '7732-18-5', '85-44-9',
+                      '67-64-1', '78-87-5', '624-72-6', '118-96-7', '124-18-5',
+                      # '526-73-8'
+                      ]
+    for CAS in fit_issue_CASs:
+        obj = VaporPressure(CASRN=CAS)
+        Ts = linspace(obj.Perrys2_8_Tmin, obj.Perrys2_8_Tmax, pts)
+        props_calc = [obj.calculate(T, DIPPR_PERRY_8E) for T in Ts]
+        res, stats = obj.fit_data_to_model(Ts=Ts, data=props_calc, model='DIPPR101',
+                              do_statistics=True, use_numba=False, multiple_tries=True, fit_method='lm')
+        assert stats['MAE'] < 1e-5
+
+@pytest.mark.slow
+@pytest.mark.fuzz
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting3_WagnerMcGarry():
+    for i, CAS in enumerate(chemicals.vapor_pressure.Psat_data_WagnerMcGarry.index):
+        obj = VaporPressure(CASRN=CAS)
+        Ts = linspace(obj.WAGNER_MCGARRY_Tmin, obj.WAGNER_MCGARRY_Tc, 10)
+        props_calc = [obj.calculate(T, WAGNER_MCGARRY) for T in Ts]
+        res, stats = obj.fit_data_to_model(Ts=Ts, data=props_calc, model='Wagner_original',
+                              do_statistics=True, use_numba=False,
+                              fit_method='lm', model_kwargs={'Tc': obj.WAGNER_MCGARRY_Tc, 'Pc': obj.WAGNER_MCGARRY_Pc})
+        assert stats['MAE'] < 1e-7
+
+@pytest.mark.slow
+@pytest.mark.fuzz
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting4_WagnerPoling():
+    for i, CAS in enumerate(chemicals.vapor_pressure.Psat_data_WagnerPoling.index):
+        obj = VaporPressure(CASRN=CAS)
+        Ts = linspace(obj.WAGNER_POLING_Tmin, obj.WAGNER_POLING_Tc, 10)
+        props_calc = [obj.calculate(T, WAGNER_POLING) for T in Ts]
+        res, stats = obj.fit_data_to_model(Ts=Ts, data=props_calc, model='Wagner',
+                              do_statistics=True, use_numba=False,
+                              fit_method='lm', model_kwargs={'Tc': obj.WAGNER_POLING_Tc, 'Pc': obj.WAGNER_POLING_Pc})
+        assert stats['MAE'] < 1e-7
+    
+@pytest.mark.slow
+@pytest.mark.fuzz
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting5_AntoinePoling():
+    for i, CAS in enumerate(chemicals.vapor_pressure.Psat_data_AntoinePoling.index):
+        obj = VaporPressure(CASRN=CAS)
+        Ts = linspace(obj.ANTOINE_POLING_Tmin, obj.ANTOINE_POLING_Tmax, 10)
+        props_calc = [obj.calculate(T, ANTOINE_POLING) for T in Ts]
+        res, stats = obj.fit_data_to_model(Ts=Ts, data=props_calc, model='Antoine',
+                              do_statistics=True, use_numba=False,
+                              multiple_tries=True,
+                              model_kwargs={'base': 10.0},  fit_method='lm')
+        assert stats['MAE'] < 1e-7
+
+@pytest.mark.slow
+@pytest.mark.fuzz
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting6_VDI_PPDS():
+    for i, CAS in enumerate(chemicals.vapor_pressure.Psat_data_VDI_PPDS_3.index):
+        obj = VaporPressure(CASRN=CAS)
+        Ts = linspace(obj.VDI_PPDS_Tm, obj.VDI_PPDS_Tc, 10)
+        props_calc = [obj.calculate(T, VDI_PPDS) for T in Ts]
+        res, stats = obj.fit_data_to_model(Ts=Ts, data=props_calc, model='Wagner',
+                              do_statistics=True, use_numba=False,
+                              fit_method='lm', model_kwargs={'Tc': obj.VDI_PPDS_Tc, 'Pc': obj.VDI_PPDS_Pc})
+        assert stats['MAE'] < 1e-7
+    
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting7_reduced_fit_params_with_jac():
+    obj = VaporPressure(CASRN='13838-16-9')
+    Tmin, Tmax = obj.WAGNER_POLING_Tmin, obj.WAGNER_POLING_Tmax
+    Ts = linspace(Tmin, Tmax, 10)
+    Ps = [obj(T) for T in Ts]
+    fit = obj.fit_data_to_model(Ts=Ts, data=Ps, model='Wagner', use_numba=False,
+                                model_kwargs={'Tc': obj.WAGNER_POLING_Tc, 'Pc': obj.WAGNER_POLING_Pc, 'd': -4.60})
+    assert fit['d'] == -4.6
+
+    fit = obj.fit_data_to_model(Ts=Ts, data=Ps, model='Wagner', use_numba=False,
+                                model_kwargs={'Tc': obj.WAGNER_POLING_Tc, 'Pc': obj.WAGNER_POLING_Pc, 'b': 2.4})
+    assert fit['b'] == 2.4
+
+    fit = obj.fit_data_to_model(Ts=Ts, data=Ps, model='Wagner', use_numba=False,
+                                model_kwargs={'Tc': obj.WAGNER_POLING_Tc, 'Pc': obj.WAGNER_POLING_Pc, 'd': -4.60, 'a': -8.329, 'b': 2.4})
+    assert fit['a'] == -8.329
+    assert fit['b'] == 2.4
+    assert fit['d'] == -4.6
+
+@pytest.mark.slow
+@pytest.mark.fuzz
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting8_TRC_AntoineExtended():
+    for i, CAS in enumerate(chemicals.vapor_pressure.Psat_data_AntoineExtended.index):
+        obj = VaporPressure(CASRN=CAS)
+        Ts = linspace(obj.ANTOINE_EXTENDED_POLING_Tmin, obj.ANTOINE_EXTENDED_POLING_Tmax, 10)
+        props_calc = [obj.calculate(T, ANTOINE_EXTENDED_POLING) for T in Ts]
+        res, stats = obj.fit_data_to_model(Ts=Ts, data=props_calc, model='TRC_Antoine_extended',
+                              do_statistics=True, use_numba=False, 
+                              fit_method='lm', model_kwargs={'Tc': obj.ANTOINE_EXTENDED_POLING_coefs[0], 
+                                                             'to': obj.ANTOINE_EXTENDED_POLING_coefs[1]})
+        assert stats['MAE'] < 1e-4
+
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting9_Yaws_Psat():
+    A, B, C, D, E, Tmin, Tmax = 53.93890302013294, -788.24, -22.734, 0.051225, 6.1896e-11, 68.15, 132.92
+    Ts = linspace(Tmin, Tmax, 10)
+    props_calc = [Yaws_Psat(T, A, B, C, D, E) for T in Ts]
+    res, stats = VaporPressure.fit_data_to_model(Ts=Ts, data=props_calc, model='Yaws_Psat',
+                          do_statistics=True, use_numba=False, 
+                          fit_method='lm')        
+    assert stats['MAE'] < 1e-5
+
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VaporPressure_fitting10_example():
+    Ts = [203.65, 209.55, 212.45, 234.05, 237.04, 243.25, 249.35, 253.34, 257.25, 262.12, 264.5, 267.05, 268.95, 269.74, 272.95, 273.46, 275.97, 276.61, 277.23, 282.03, 283.06, 288.94, 291.49, 293.15, 293.15, 293.85, 294.25, 294.45, 294.6, 294.63, 294.85, 297.05, 297.45, 298.15, 298.15, 298.15, 298.15, 298.15, 299.86, 300.75, 301.35, 303.15, 303.15, 304.35, 304.85, 305.45, 306.25, 308.15, 308.15, 308.15, 308.22, 308.35, 308.45, 308.85, 309.05, 311.65, 311.85, 311.85, 311.95, 312.25, 314.68, 314.85, 317.75, 317.85, 318.05, 318.15, 318.66, 320.35, 320.35, 320.45, 320.65, 322.55, 322.65, 322.85, 322.95, 322.95, 323.35, 323.55, 324.65, 324.75, 324.85, 324.85, 325.15, 327.05, 327.15, 327.2, 327.25, 327.35, 328.22, 328.75, 328.85, 333.73, 338.95]
+    Psats = [58.93, 94.4, 118.52, 797.1, 996.5, 1581.2, 2365, 3480, 3893, 5182, 6041, 6853, 7442, 7935, 9290, 9639, 10983, 11283, 13014, 14775, 15559, 20364, 22883, 24478, 24598, 25131, 25665, 25931, 25998, 26079, 26264, 29064, 29598, 30397, 30544, 30611, 30784, 30851, 32636, 33931, 34864, 37637, 37824, 39330, 40130, 41063, 42396, 45996, 46090, 46356, 45462, 46263, 46396, 47129, 47396, 52996, 52929, 53262, 53062, 53796, 58169, 59328, 66395, 66461, 67461, 67661, 67424, 72927, 73127, 73061, 73927, 79127, 79527, 80393, 79927, 80127, 81993, 80175, 85393, 85660, 85993, 86260, 86660, 92726, 92992, 92992, 93126, 93326, 94366, 98325, 98592, 113737, 136626]
+    res, stats = TDependentProperty.fit_data_to_model(Ts=Ts, data=Psats, model='Antoine', do_statistics=True, multiple_tries=True)
+    assert stats['MAE'] < 0.014
 
 @pytest.mark.meta_T_dept
 def test_VaporPressure_analytical_derivatives():
