@@ -20,7 +20,7 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.'''
 
-from fluids.numerics import assert_close, assert_close1d, assert_close2d
+from fluids.numerics import assert_close, assert_close1d, assert_close2d, linspace
 import numpy as np
 import pytest
 import json
@@ -28,9 +28,11 @@ import pandas as pd
 from thermo.volume import *
 from thermo.volume import VDI_TABULAR
 from thermo.eos import *
+import chemicals
 from chemicals.utils import Vm_to_rho, zs_to_ws
 from thermo.vapor_pressure import VaporPressure
 from thermo.utils import POLY_FIT
+from chemicals.volume import *
 from thermo.volume import LALIBERTE, COSTALD_MIXTURE_FIT, RACKETT_PARAMETERS, COSTALD_MIXTURE, LINEAR, RACKETT
 from thermo.volume import HTCOSTALD, COOLPROP, DIPPR_PERRY_8E, VDI_TABULAR, RACKETTFIT, YEN_WOODS_SAT, BHIRUD_NORMAL, VDI_PPDS, TOWNSEND_HALES, HTCOSTALDFIT, CAMPBELL_THODOS, MMSNM0, RACKETT, YAMADA_GUNN
 from thermo.volume import PITZER_CURL, EOS, TSONOPOULOS_EXTENDED, ABBOTT, COOLPROP, IDEAL, TSONOPOULOS, CRC_VIRIAL
@@ -331,6 +333,149 @@ def test_VolumeSolid_works_with_no_data():
     # Test pentane has no property but that it succeeds
     VolumeSolid(CASRN='109-66-0').T_dependent_property(300)
 
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VolumeSolid_fitting2():
+    obj = VolumeSolid(CASRN='7782-44-7', load_data=False)
+    Ts_gamma = [42.801, 44.0, 46.0, 48.0,
+               50.0, 52.0, 54.0, 54.361]
+    
+    Vms_gamma = [23.05e-6, 23.06e-6, 23.18e-6, 23.30e-6,
+                23.43e-6, 23.55e-6, 23.67e-6, 23.69e-6]
+    obj.fit_add_model(Ts=Ts_gamma, data=Vms_gamma, model='DIPPR100', name='gamma')
+    assert obj.method == 'gamma'
+
+@pytest.mark.meta_T_dept
+@pytest.mark.fitting
+def test_VolumeSolid_fitting1():
+    # Initial Result: went poorly + Don't have that exact model
+    # 1) A special polynomial fitting may be useful
+    # 2) If the model does not "converge", it may be worth trying other solvers
+    ammonia_Ts = [194.150, 194.650, 195.150, 195.650, 196.150, 196.650, 197.150, 197.650, 198.150, 198.650, 199.150, 199.650, 200.150, 200.650, 201.150, 201.650, 202.150, 202.650, 203.150, 203.650, 204.150]
+    ammonia_Vms = [1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3, 1/47.9730E3]
+    obj = VolumeSolid(CASRN='7664-41-7', load_data=False)
+    
+    res, stats = VolumeSolid.fit_data_to_model(Ts=ammonia_Ts, data=ammonia_Vms, model='DIPPR100',
+                          do_statistics=True, use_numba=False)
+    assert stats['MAE'] < 1e-8
+
+
+
+
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VolumeSolid_fitting0():
+    Ts_alpha = [4.2, 18.5, 20, 22, 23.880]
+    Vms_alpha = [20.75e-6, 20.75e-6, 20.75e-6, 20.78e-6, 20.82e-6]
+    
+    Ts_beta = [23.880, 24, 26, 28,
+               30, 32, 34, 36,
+               38, 40, 42, 43.801]
+    Vms_beta = [20.95e-6, 20.95e-6, 21.02e-6, 21.08e-6,
+                21.16e-6, 21.24e-6, 21.33e-6, 21.42e-6, 
+                21.52e-6, 21.63e-6, 21.75e-6, 21.87e-6]
+                
+    Ts_gamma = [42.801, 44.0, 46.0, 48.0,
+               50.0, 52.0, 54.0, 54.361]
+    
+    Vms_gamma = [23.05e-6, 23.06e-6, 23.18e-6, 23.30e-6,
+                23.43e-6, 23.55e-6, 23.67e-6, 23.69e-6]
+    
+    obj = VolumeSolid(CASRN='7782-44-7', load_data=False)
+    # obj.add_tabular_data(Ts=Ts_alpha, properties=Vms_alpha, name='O2_alpha')
+    # obj.add_tabular_data(Ts=Ts_beta, properties=Vms_beta, name='O2_beta')
+    # obj.add_tabular_data(Ts=Ts_gamma, properties=Vms_gamma, name='O2_gamma')
+    
+    fit_zeros_specified = obj.fit_data_to_model(Ts=Ts_gamma, data=Vms_gamma, model='DIPPR100', do_statistics=False, use_numba=False,
+                     model_kwargs={'C': 0.0, 'D': 0.0, 'E': 0.0, 'F': 0.0, 'G': 0.0, 'B': 0.0})
+    for l in ('B', 'C', 'D', 'E', 'F'):
+        assert fit_zeros_specified[l] == 0.0
+        
+    fit_constant = obj.fit_data_to_model(Ts=Ts_gamma, data=Vms_gamma, model='constant', do_statistics=False, use_numba=False)
+    assert_close(fit_zeros_specified['A'], fit_constant['A'], rtol=1e-13)
+
+@pytest.mark.meta_T_dept
+@pytest.mark.fitting
+def test_VolumeLiquid_fitting0():
+
+    ammonia_Ts_V_l = [195, 206.061, 217.121, 228.182, 239.242, 239.82, 250.303, 261.363, 272.424, 283.484, 294.545, 305.605, 316.666, 327.726, 338.787, 349.847, 360.908, 371.968, 383.029, 394.089, 405.15]
+    ammonia_V_ls = [1/43200.2, 1/42436.8, 1/41658, 1/40862.4, 1/40048.1, 1/40005, 1/39213, 1/38354.7, 1/37470.3, 1/36556, 1/35607.3, 1/34618.5, 1/33582.2, 1/32488.1, 1/31322.5, 1/30064.9, 1/28683.4, 1/27123.2, 1/25274.3, 1/22840.2, 1/16984.9]
+    ammonia_rhom_ls = [1/v for v in ammonia_V_ls]
+    obj = VolumeLiquid(CASRN='7664-41-7', load_data=False)
+
+    coeffs, stats = obj.fit_data_to_model(Ts=ammonia_Ts_V_l, data=ammonia_rhom_ls, model='DIPPR105',
+                          do_statistics=True, use_numba=False,
+                          guesses= {'A': 4.0518E3, 'B': 0.27129, 'C': 405.4,'D': 0.31349})
+    assert stats['MAE'] < 1e-5
+    
+@pytest.mark.meta_T_dept
+@pytest.mark.fitting
+def test_VolumeLiquid_fitting1_dippr():
+    fit_check_CASs = ['124-38-9', '74-98-6', '1333-74-0', '630-08-0', 
+                      '100-21-0', '624-92-0', '624-72-6', '74-86-2',
+                      '115-07-1', '64-18-6']
+    for CAS in fit_check_CASs:
+        obj = VolumeLiquid(CASRN=CAS)
+        Ts = linspace(obj.DIPPR_Tmin, obj.DIPPR_Tmax, 8)
+        props_calc = [1.0/obj.calculate(T, DIPPR_PERRY_8E) for T in Ts]
+    
+        res, stats = obj.fit_data_to_model(Ts=Ts, data=props_calc, model='DIPPR105',
+                              do_statistics=True, use_numba=False, fit_method='lm')
+        assert stats['MAE'] < 1e-5
+
+@pytest.mark.slow
+@pytest.mark.fuzz
+@pytest.mark.fitting
+@pytest.mark.meta_T_dept
+def test_VolumeLiquid_fitting2_dippr_116_ppds():
+    for i, CAS in enumerate(chemicals.volume.rho_data_VDI_PPDS_2.index):
+        obj = VolumeLiquid(CASRN=CAS)
+        Ts = linspace(obj.T_limits[VDI_PPDS][0], obj.T_limits[VDI_PPDS][1], 8)
+        props_calc = [Vm_to_rho(obj.calculate(T, VDI_PPDS), obj.VDI_PPDS_MW) for T in Ts]
+        
+        res, stats = obj.fit_data_to_model(Ts=Ts, data=props_calc, model='DIPPR116',
+                              do_statistics=True, use_numba=False, fit_method='lm', 
+                              model_kwargs={'Tc': obj.VDI_PPDS_Tc, 'A': obj.VDI_PPDS_rhoc})
+        assert stats['MAE'] < 1e-7
+
+@pytest.mark.meta_T_dept
+@pytest.mark.fitting
+def test_VolumeLiquid_fitting3():
+    # From yaws
+    Tc, rhoc, b, n, MW = 627.65, 433.128, 0.233, 0.2587, 66.0
+    Ts = linspace(293.15, 298.15, 10)
+    props_calc = [Rackett_fit(T, Tc, rhoc, b, n, MW) for T in Ts]
+    res, stats = VolumeLiquid.fit_data_to_model(Ts=Ts, data=props_calc, model='Rackett_fit',
+                          do_statistics=True, use_numba=False, model_kwargs={'MW':MW, 'Tc': Tc,},
+                          fit_method='lm')
+    assert stats['MAE'] < 1e-5
+    
+    # From yaws
+    Tc, rhoc, b, n, MW = 1030.0, 1795.0521319999998, 0.96491407, 0.15872, 97.995
+    Ts = linspace(315.51, 393.15, 10)
+    props_calc = [Rackett_fit(T, Tc, rhoc, b, n, MW) for T in Ts]
+    res, stats = VolumeLiquid.fit_data_to_model(Ts=Ts, data=props_calc, model='Rackett_fit',
+                          do_statistics=True, use_numba=False, model_kwargs={'MW':MW, 'Tc': Tc,},
+                          fit_method='lm')
+    assert stats['MAE'] < 1e-5
+
+    # From yaws
+    Tc, rhoc, b, n, MW = 545.03, 739.99, 0.3, 0.28571, 105.921
+    Ts = linspace(331.15, 332.9, 10)
+    props_calc = [Rackett_fit(T, Tc, rhoc, b, n, MW) for T in Ts]
+    res, stats = VolumeLiquid.fit_data_to_model(Ts=Ts, data=props_calc, model='Rackett_fit',
+                          do_statistics=True, use_numba=False, model_kwargs={'MW':MW, 'Tc': Tc,},
+                          fit_method='lm')
+    assert stats['MAE'] < 1e-5
+
+    # From yaws
+    Tc, rhoc, b, n, MW = 1800.0, 2794.568007, 0.647077183, 0.8, 98.999
+    Ts = linspace(703.0, 1620., 10)
+    props_calc = [Rackett_fit(T, Tc, rhoc, b, n, MW) for T in Ts]
+    res, stats = VolumeLiquid.fit_data_to_model(Ts=Ts, data=props_calc, model='Rackett_fit',
+                          do_statistics=True, use_numba=False, model_kwargs={'MW':MW, 'Tc': Tc,},
+                          fit_method='lm')
+    assert stats['MAE'] < 1e-5
 
 
 @pytest.mark.meta_T_dept
